#-----------------------------------------------------
# support define and bind variable dynamically
#-----------------------------------------------------
$dyn_vars = nil

# set value to a variable
def set_var(var_name, var_value)
  if $dyn_vars == nil
    $dyn_vars = binding
  end

  strEval = var_name + "=" + var_value
  eval strEval, $dyn_vars
end

def check_dynamic_value value
  if !value.is_a? Fixnum
    if value.include? "params="
      resolve_params value
    else
      bind_with_dyn_vars value
    end
  else
    value
  end
end


def resolve_params url
  condition = url.match(/params='([^']*)'/)[0]
  if condition
    params_name = url.match(/params='([^']*)'/)[1]
    params_value = $PARAMS["#{params_name.downcase}"].to_s.strip
    if params_value.present?
      url = url.gsub(condition, params_value)
    else
      raise "ERROR: no data found with given params #{params_name}"
    end
  end
  url
end

# bind string with $dyn_vars context
def bind_with_dyn_vars(str)
  if $dyn_vars == nil
    $dyn_vars = binding
  end

  strEval = '"' + str + '"'
  return eval strEval, $dyn_vars
end

# evaluate operation/statement with $dyn_vars context
def eval_with_dyn_vars(operation)
  if $dyn_vars == nil
    $dyn_vars = binding
  end

  eval operation, $dyn_vars
end

def bind_with_dyn_json_vars(json, bind_json)
  if json.kind_of? Hash
    json.each_pair do |k, v|
      if v.kind_of? String
        bind_json[bind_with_dyn_vars(k)] = bind_with_dyn_json_vars(v, bind_json)
      elsif v.kind_of? Hash
        temp = Hash.new
        v.each_pair do |k1, v1|
          temp[bind_with_dyn_vars(k1)] = bind_with_dyn_json_vars(v1, temp)
        end
        bind_json[bind_with_dyn_vars(k)] = temp
      elsif v.kind_of? Array
        temp1 = Array.new
        v.each { |item|
          temp2 = Hash.new
          bind_with_dyn_json_vars(item, temp2)
          temp1 << temp2
        }
        bind_json[bind_with_dyn_vars(k)] = temp1
      end
    end
  elsif json.kind_of? Array
    temp1 = Array.new
    json.each { |item|
      temp2 = Hash.new
      bind_with_dyn_json_vars(item, temp2)
      temp1 << temp2
    }
    return temp1
  else
    return bind_with_dyn_vars(json)
  end
end


def var_collect string_var
  if string_var =~ /^.*{year}.*$/
    string_var = replace_year(string_var)
  end

  if string_var =~ /^.*{month}.*$/
    string_var = replace_month(string_var)
  end

  if string_var =~ /^.*{day}.*$/
    string_var = replace_day(string_var)
  end

  if string_var =~ /^.*{store_value}.*$/
    string_var = $context_value
  end

  return string_var
end

def replace_year str
  t = Time.now()
  return str.gsub("{year}", t.year.to_s)
end

def replace_month str
  t = Time.now()
  _month = t.month
  if _month < 10
    return str.gsub("{month}", "0#{_month.to_s}")
  else
    return str.gsub("{month}", "#{_month.to_s}")
  end
  return str
end

def replace_day str
  t = Time.now()
  _day = t.day
  if _day < 10
    return str.gsub("{day}", "0#{_day.to_s}")
  else
    return str.gsub("{day}", "#{_day.to_s}")
  end
  return str
end

def replace_pipe str_pipe
  put_log ">>>> #{str_pipe}"
  if str_pipe =~ /^.*{pipe}.*$/
    return str_pipe.gsub("{pipe}", "|")
  end
  return str_pipe
end

def store_string_as_variable(string,variable)
  # $context_value = bind_with_dyn_vars(string)
  # set_var(variable, '$context_value')
  require 'erb'
  ERB.new(string, 0, "", "#{variable}").result(binding)
end

def print_variable(variable)
  puts "VALUE OF #{variable}: #{eval_with_dyn_vars(variable)}"
end
