Before do
  $driver = getDriver
  $wait_time = ENV["WAIT_TIME"] || 30
  $print_log = ENV["PRINT_LOG"] || false
end

AfterConfiguration do
  $OBJECT = loadYAMLfile("features/step_definitions/repositories/project_object.yml")
  if ENV['SERVER_URL'].nil?
    puts "❌  ERROR: Missing param SERVER_URL."
    exit(1)
  end
  if ENV['BROWSER'].nil?
    puts "❌  ERROR: Missing param BROWSER."
    exit(1)
  end
  if ENV['DELETE_COOKIE']
    browser = Capybara.current_session.driver.browser
    browser.manage.delete_all_cookies
  end
end

def getDriver
  case ENV['BROWSER']
  when "chrome"
    @chrome_options = loadYAMLfile('config/chrome_options.yaml')
    Capybara.register_driver :selenium do |app|
       Capybara::Selenium::Driver.new(app, :url => ENV['SERVER_URL'],:desired_capabilities => Selenium::WebDriver::Remote::Capabilities.chrome(chromeOptions: @chrome_options))
    end
    Capybara.current_driver = :selenium
  when "firefox"
    @firefox_options = loadYAMLfile('config/firefox_options.yaml')
    Capybara.register_driver :selenium do |app|
      Capybara::Selenium::Driver.new(app, :url => ENV['SERVER_URL'], :desired_capabilities => Selenium::WebDriver::Remote::Capabilities.firefox(firefoxOptions: @firefox_options))
    end
    Capybara.current_driver = :selenium
  when "safari"
    @safari_options = loadYAMLfile('config/safari_options.yaml')
    Capybara.register_driver :selenium do |app|
      Capybara::Selenium::Driver.new(app, :url => ENV['SERVER_URL'], :desired_capabilities => Selenium::WebDriver::Remote::Capabilities.safari(safariOptions: @safari_options))
    end
    Capybara.current_driver = :selenium
  when "ie"
    @ie_options = loadYAMLfile('config/ie_options.yaml')
    Capybara.register_driver :selenium do |app|
      Capybara::Selenium::Driver.new(app, :url => ENV['SERVER_URL'], :desired_capabilities => Selenium::WebDriver::Remote::Capabilities.internet_explorer(ieOptions: @ie_options))
    end
    Capybara.current_driver = :selenium
  when "chrome_headless"
    @chrome_options = loadYAMLfile('config/chrome_options.yaml')
    Capybara.register_driver :selenium do |app|
      Capybara::Selenium::Driver.new(app, :desired_capabilities => Selenium::WebDriver::Remote::Capabilities.chrome(chromeOptions: @chrome_options))
    end
    Capybara.current_driver = :selenium_headless
  when "firefox_headless"
    @firefox_options = loadYAMLfile('config/firefox_options.yaml')
    Capybara.register_driver :selenium do |app|
      Capybara::Selenium::Driver.new(app, :desired_capabilities => Selenium::WebDriver::Remote::Capabilities.firefox(firefoxOptions: @firefox_options))
    end
    Capybara.current_driver = :selenium_headless
   else
    puts "❌  ERROR: Unsupported browser: #{ENV['BROWSER']}. Available: chrome || firefox || safari || ie || chrome_headless || firefox_headless"
    exit(1)
  end
end

def loadYAMLfile(filename)
  begin
    return YAML.load_file(filename)
  rescue Psych::SyntaxError => ex
    puts "❌ SyntaxError when reading file: #{ex}"
    exit(1)
  end
end

def validateReport
  if ENV['TESTSUITE'] && ENV['TESTCASE_REPORT_NAME']
    @report_path = "Reports/#{ENV['TESTSUITE']}"
    @testcase_report_name = ENV['TESTCASE_REPORT_NAME']
  else
    puts "❌ ERROR: Missing param TESTSUITE || TESTCASE_REPORT_NAME. Report could not be generated."
    exit(1)
  end
end

After do |scenario|
  #   save_screenshot = ENV['SCREENSHOT'] || false
  #   if save_screenshot == true
  #     file_name = 'failed_%s.png' % rand(1000).to_s
  #     page.save_screenshot($screenshot+file_name) if scenario.failed?
  #     begin
  #       Capybara.page.driver.quit
  #       Capybara.send(:session_pool).delete_if { |key, value| key =~ /selenium/i }
  #     rescue StandardError => myStandardError
  #       puts "❌  Error: #{myStandardError}"
  #       exit(1)
  #     end
  #   end
  # $browser_data = page.driver.browser.capabilities
  # p "$browser_data:#{$browser_data.inspect}"
end

at_exit do
  if ENV['REPORT']
    validateReport
    time = Time.now.getutc
    if $browser_data
      browser_name = $browser_data[:browser_name]
      browser_version = $browser_data[:browser_version]
      platform = $browser_data[:platform_name]
    end
    ReportBuilder.configure do |config|
      config.report_path = "#{@report_path}/#{@testcase_report_name}"
      config.input_path = "#{@report_path}/#{@testcase_report_name}.json"
      config.report_types = [:json, :html]
      config.color = 'blue'
      config.additional_info = {
        'Browser' => browser_name || nil,
        'Platform' => platform || nil,
        'Browser Version' => browser_version || nil,
        'Report Generated' => time
      }
    end
    options = { report_title: "INFOdation Test Management System" }
    ReportBuilder.build_report options
  end
end
